var RestEngine = function(ZhLogger, $http, templateManager, ScreenService, ZhFeatures, ZHPromise, EventAggregator, requestInterceptingEngine) {
	var _http = $http;
	var _enumerationRequest = 0;
	var token;
	var buildHTTPParams = function(requestParams, baseUrl) {
		if (!requestParams.baseUrl) {
			requestParams.baseUrl = baseUrl;
		}
		return {
			timeout: requestParams.timeout,
			url: requestParams.baseUrl + requestParams.service,
			dataType: "json",
			method: requestParams.typeRequest,
			data: ZhFeatures.usePostData ? $.param(requestParams.params) : requestParams.params,
			headers: {
				'Content-Type': ZhFeatures.usePostData ? 'application/x-www-form-urlencoded' : 'application/json',
				'User-Id': localStorage.getItem("USER_ID"),
				'Zeedhi-Token': token
			},
			withCredentials: true
		};
	};

	var debouncedOnRequestIdleEvent;
	templateManager.projectPromise.then(function(){
		if(templateManager.project.onRequestIdleTimeout){
			var timeout = templateManager.project.onRequestIdleTimeout;
			var callOnRequestIdleEvent = function() {
				var lastRequest = localStorage.getItem('OnRequestIdleEvent_LastRequest');
				var now = Date.now();
				var difference = now - lastRequest;
				if(lastRequest && difference < timeout){
					setTimeout(callOnRequestIdleEvent, difference);
				}
				else{
					EventAggregator.onRequestIdle();
				}
			};
			debouncedOnRequestIdleEvent = Util.buildDebounceMethod(callOnRequestIdleEvent, timeout);
		}

		if(templateManager.project.useAntiCSRF){
			var antiCSRFRoute = templateManager.project.antiCSRFRoute;
			var baseUrl = getBaseUrl(antiCSRFRoute, null, templateManager.getMappedServicePackages());
			var serviceName = clearJsonName(antiCSRFRoute);
			_http({
				method: "GET",
				url: baseUrl + serviceName,
				headers: {
					'Content-Type': 'application/json',
					'User-Id': localStorage.getItem("USER_ID"),
				},
				withCredentials: true
			}).then(function(response){
				var tokenValue = Util.getNestedProperty(response, ['data', 'dataset', 'zeedhi-dataset-token', 'token']);
				token = tokenValue || null;
			}).catch(function(err){
				token = null;
				ZhLogger.error("An error occurred when was building a token.");
			});
		}
	});

	var _doRequest = function(requestParams, requestCount, deferred, index) {
		index = index || 0;

		var baseUrls = Util.parseToArray(requestParams.baseUrls);
		if (baseUrls[index]) {
			var baseUrl = baseUrls[index];
			var params = buildHTTPParams(requestParams, baseUrl);

			//requestStart
			var currentRequest = _enumerationRequest++;
			EventAggregator.onRequestStart({
				data: requestParams,
				index: currentRequest,
				requestConfig: params
			});
			localStorage.setItem('OnRequestIdleEvent_LastRequest', Date.now());
			var successCallback = function(data) {
				if (requestParams.callBack) {
					requestParams.callBack(data);
				}
				if(data.error) {
					data.data = Util.clone(data);
					deferred.reject(data);
				} else {
					deferred.resolve(data);
				}
				EventAggregator.onRequestSuccess({
					data: data,
					index: currentRequest
				});
			};

			var errorCallback = function(error) {
				requestCount++;
				var isToRetryAgain = requestCount < requestParams.returnCount;
				if (isToRetryAgain || baseUrls[index + 1]) {
					if (requestParams.retryCallback) {
						requestParams.retryCallback(error);
					}
					EventAggregator.onRequestRetry({
						data: error,
						index: currentRequest,
						retryCount: requestParams.returnCount,
						requestCount: requestCount,
						baseUrl: params.baseUrl
					});
					if (!isToRetryAgain) {
						requestCount = 0;
						++index;
						delete requestParams.baseUrl;
					}
					_doRequest(requestParams, requestCount, deferred, index);
				} else {
					if (requestParams.errorCallBack) {
						requestParams.errorCallBack(error);
					}
					ZhLogger.error("At: " + this.params.url);
					if (error && error.message && error.message.indexOf("Unexpected token") !== -1) {
						ZhLogger.error("Returned an invalid JSON");
						ZhLogger.error("See your network requests");
					} else {
						ZhLogger.error("Said: " + error.message);
					}
					deferred.reject(error);
					EventAggregator.onRequestError({
						data: error,
						index: currentRequest
					});
				}
			};

			if(debouncedOnRequestIdleEvent){
				debouncedOnRequestIdleEvent();
			}

			requestInterceptingEngine.intercept(params)
			.then(function () {
				_http(params)
				.success(successCallback)
				.then(null, errorCallback.bind({ params: params }));
			});

		} else {
			ZhLogger.error("Invalid index of serviceUrls source");
			deferred.reject({
				message: "Invalid index of serviceUrls source"
			});
		}
	};

	/*
	 *@params (service, params, callBack, typeRequest, errorCallBack, baseUrl, timeout, retryCount, retryCallback, showLoader)
	 *
	 */
	this.request = function() {
		var deferred = ZHPromise.defer();
		var requestParams = {};
		var requestCount = 0;
		var showLoader;

		if (Util.isObject(arguments[0])) {
			requestParams = arguments[0];
			requestParams.timeout = requestParams.timeout || templateManager.project.timeout;
			showLoader = arguments[1];
		} else {
			requestParams.service = arguments[0];
			requestParams.params = arguments[1];
			requestParams.callBack = arguments[2];
			requestParams.typeRequest = arguments[3] || RequestType.POST;
			requestParams.errorCallBack = arguments[4];
			requestParams.baseUrl = arguments[5];
			requestParams.timeout = arguments[6] || templateManager.project.timeout;
			requestParams.returnCount = arguments[7] || 0;
			requestParams.retryCallback = arguments[8];
			showLoader = arguments[9];
		}

		if (showLoader === undefined) {
			showLoader = requestParams.params && (requestParams.params.requestType === 'DataSet' || requestParams.params.requestType === 'Row');
		}
		if (showLoader) ScreenService.showLoader();
		requestParams.baseUrls = getBaseUrl(requestParams.service, requestParams.baseUrl, templateManager.getMappedServicePackages());
		requestParams.service = clearJsonName(requestParams.service);

		_doRequest(requestParams, requestCount, deferred);
		return deferred.then(function(data) {
			if (showLoader) ScreenService.hideLoader();
			return data;
		}, function(response) {
			if (showLoader) ScreenService.hideLoader();
			return ZHPromise.reject(response);
		});
	};

	var _requestMetaData = function(service, dataType, typeRequest, xsrf, requestParams, callBack, sourceUrls, index, deferred) {
		index = parseInt(index);
		if (isNaN(index) || index < 0) {
			index = 0;
		}
		if (!deferred) {
			deferred = ZHPromise.defer();
		}

		sourceUrls = Util.parseToArray(sourceUrls);
		if (sourceUrls[index]) {
			var url = sourceUrls[index] + service;
			var params = {
				url: url,
				dataType: dataType,
				method: typeRequest,
				data: xsrf,
				headers: {
					'Content-Type': 'application/x-www-form-urlencoded',
					'User-Id': localStorage.getItem("USER_ID")
				}
			};
			EventAggregator.onRequestMetaDataStart({
				data: requestParams,
				requestConfig: params
			});
			_http(params).success(function(data) {
				if (callBack) {
					callBack(data);
				}
				deferred.resolve({
					data: data,
					url: url
				});
			}).then(null, function(error) {
				if ((sourceUrls.length - 1) > index) {
					_requestMetaData(service, dataType, typeRequest, xsrf, requestParams, callBack, sourceUrls, (index + 1), deferred);
				} else {
					ZhLogger.error(error);
					ZhLogger.error("At: " + this.params.url);
					ZhLogger.error("Message: " + error.message);
					deferred.reject(error);
				}
			}.bind({
				params: requestParams
			}));
		} else {
			ZhLogger.error("Invalid index of metadata source");
			deferred.reject({
				message: "Invalid index of metadata source"
			});
		}
		return deferred;
	};

	this.requestMetaData = function(service, params, callBack, baseUrl) {
		params = params || {};
		var xsrf = ZhFeatures.usePostData ? $.param(params) : params;
		return _requestMetaData(clearJsonName(service), 'json', RequestType.GET, xsrf, params, callBack, getBaseUrl(service, baseUrl, templateManager.getMappedPackages()));
	};

	function getBaseUrl(service, baseUrl, mappedPackages) {
		if (!baseUrl) {
			var pckg = service.split("#");
			baseUrl = pckg.length > 1 ? mappedPackages[pckg[0]] : mappedPackages.default;
		}
		return baseUrl;
	}

	function clearJsonName(jsonName) {
		var pckg = jsonName.split("#");
		return pckg.length > 1 ? pckg[1] : pckg[0];
	}

	this.post = this.request;
	this.delete = function(service, params, callBack, errorCallBack, baseUrl, timeout) {
		return this.request(service, params, callBack, RequestType.DELETE, errorCallBack, baseUrl, timeout);
	};
	this.zhdelete = this.delete;//Old sugar method
	this.put = function(service, params, callBack, errorCallBack, baseUrl, timeout) {
		return this.request(service, params, callBack, RequestType.PUT, errorCallBack, baseUrl, timeout);
	};
	this.get = function(service, params, callBack, errorCallBack, baseUrl, timeout) {
		return this.request(service, params, callBack, RequestType.GET, errorCallBack, baseUrl, timeout);
	};

	this.registerInterceptor = function registerInterceptor(interceptorConfiguration) {
		return requestInterceptingEngine.registerInterceptor(interceptorConfiguration);
	};
	this.deregisterInterceptor = function deregisterInterceptor(interceptorIndex) {
		return requestInterceptingEngine.deregisterInterceptor(interceptorIndex);
	};
	this.getInterceptors = requestInterceptingEngine.getInterceptors;
};