(function() {
	var ZeedhiDirectives = angular.module("ZeedhiDirectives");

	function positionTooltip(element, tooltipContent) {
		var bottom = 'auto',
			top = element.offset().top + element.innerHeight() + 2,
			left = element.offset().left,
			right = 'auto';

		if (element.offset().top + element.innerHeight() + tooltipContent.innerHeight() + 5 > window.innerHeight) {
			bottom = window.innerHeight - element.offset().top + 2;
			top = 'auto';
		}

		if (element.offset().left + tooltipContent.innerWidth() + 5 > window.innerWidth) {
			left = 'auto';
			right = 5;
		}

		tooltipContent.css({
			bottom: bottom,
			top: top,
			left: left,
			right: right
		});
	}

	ZeedhiDirectives.directive('zhTitleTooltip', function() {
		var tooltipContent;
		return {
			link: function($scope, element, attrs, controller) {
				function hideTooltip() {
					element.off('remove');
					tooltipContent.hide();
				}

				element.hover(function() {
					tooltipContent = tooltipContent || $('#tooltip-content');
					if (attrs.zhTitleTooltip) {
						tooltipContent.html(attrs.zhTitleTooltip);

						positionTooltip(element, tooltipContent);

						// prevent tooltip staying on if element is removed before hover out
						element.on('remove', hideTooltip);

						tooltipContent.show();
					}
				}, hideTooltip);
			}
		};
	});

	ZeedhiDirectives.directive('zhSelectMultipleTooltip', function() {
		var tooltipContent;
		return {
			link: function($scope, element, attrs, controller) {
				function hideTooltip() {
					element.off('remove');
					tooltipContent.hide();
				}

				element.hover(function() {
					var text = element.html();
					tooltipContent = tooltipContent || $('#tooltip-content');
					if(text){
						tooltipContent.html(text);
						positionTooltip(element, tooltipContent);

						// prevent tooltip staying on if element is removed before hover out
						element.on('remove', hideTooltip);
						tooltipContent.show();
					}
				}, hideTooltip);
			}
		};
	});

	ZeedhiDirectives.directive('zhTooltipOverflow', function() {
		var tooltipContent;
		return {
			link: function($scope, element, attrs, controller) {
				function hideTooltip() {
					element.off('remove');
					tooltipContent.hide();
				}

				element.hover(function() {
					var parent = element.parent()[0];
					tooltipContent = tooltipContent || $('#tooltip-content');

					if (parent.clientWidth < parent.scrollWidth) { // has overflow
						tooltipContent.html(element.html());

						positionTooltip(element, tooltipContent);

						// prevent tooltip staying on if element is removed before hover out
						element.on('remove', hideTooltip);

						tooltipContent.show();
					}
				}, hideTooltip);
			}
		};
	});

	ZeedhiDirectives.directive('zhMenuTooltip', function() {
		return {
			link: function($scope, element, attrs, controller) {
				function hideTooltip() {
					element.off('remove');
					$('#menu-tooltip-content').hide();
				}

				element.hover(function() {
					if (element[0].scrollWidth > element[0].offsetWidth) {
						var tooltipContent = $('#menu-tooltip-content');
						tooltipContent.html(element.html());

						var bottom = 'auto',
							top = element.offset().top - 20,
							left = element.offset().left + parseInt(element.css('padding-left')) + 10,
							right = 'auto';

						tooltipContent.css({
							bottom: bottom,
							top: top,
							left: left,
							right: right
						});

						// prevent tooltip staying on if element is removed before hover out
						element.on('remove', hideTooltip);

						tooltipContent.show();
					}
				}, hideTooltip);
			}
		};
	});
})();